@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: approve.c,v 1.150 2007/03/07 15:00:16 will Exp $";
+static const char rcsid[] = "$Id: approve.c,v 1.153 2007/12/20 16:26:38 will Exp $";
 
 /* The code should compile with either ANSI C or K&R compilers. */
 
@@ -670,7 +670,7 @@ char *usrcmd;
 {
     static char *usage[] = {
 "",
-"  %s command [args...]   ......   executes command [args].",
+"  %s [-r path] command [args...]  executes command [args].",
 "",
 "  %s   ........................   basic listing of permitted cmds.",
 "  %s [-H | -f] [-S] [MasqOpts]   list ok cmds: -H=verbose,-f=just-facts.",
@@ -687,7 +687,11 @@ char *usrcmd;
 "    -D   Same as -d, plus tell more about variables, plus other details.",
 "    -f   (just the facts, ma'm): lists your cmds as: Cmd FullPath [args]",
 "	      Useful for scripts that want a list of what you may execute.",
+"    -r path -- error if command doesn't expand to invoking <path>.",
 "    -S -- force stdin to be used for reading passwords.",
+"    -t -- test mode: do all checks excepting user input; generate normal",
+"	      output (no special debug output); do not not execute anything.",
+"	      Exit code=0 if ok to execute, else 1."
 "",
 "MasqueradeOptions:",
 "    ** No command is ever executed when any MasqueradeOption is used.**",
