@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: checks.c,v 1.180 2004/10/22 16:00:45 will Exp $";
+static const char rcsid[] = "$Id: checks.c,v 1.190 2007/03/07 15:04:41 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -11,10 +11,114 @@ static const char rcsid[] = "$Id: checks.c,v 1.180 2004/10/22 16:00:45 will Exp
 
 #include "super.h"
 #include "version.h"
+#include "s_hsearch.h"
+
+#define MAX_KEYLEN 1000            /* Large enough? */
+static char keybuf[MAX_KEYLEN];
+
+static char *YES = "Y";
+static char *NO  = "N";
 
 #ifdef HAVE_INNETGR
+/*
 #define netgrp_u_compare(pattern, user) innetgr(pattern, NULL, user, NULL)
 #define netgrp_h_compare(pattern, host) innetgr(pattern, host, NULL, NULL)
+ */
+
+static int created_user_table = 0;
+static netgrp_u_compare(pattern, user)
+  char *pattern;
+  char *user;
+{
+    ENTRY item, *found_item;
+    int passed, klen;
+
+    if (!created_user_table) {
+        if (!s_hcreate(HS_USER, 1000)) {
+	    return Error(0, 0,
+		"%tCouldn't allocate hash table for user ng processing\n");
+	}
+        created_user_table = 1;
+    }
+    /* Catenate pattern and user for the key such that
+     * "abc" + "de" is not the same as "ab" + "cde".
+     * Use "\n", as that will have been stripped from input before we get here.
+     */
+    if (snprintf(keybuf, MAX_KEYLEN, "%s\n%s", pattern, user) >= MAX_KEYLEN) {
+         return Error(0, 0,
+             "%tpattern and user too large for user ng processing\n");
+    }
+
+    item.key = keybuf;
+    if (found_item = s_hsearch(HS_USER, item, FIND)) {
+        return found_item->data == YES? 1: 0;
+    }
+    passed = innetgr(pattern, NULL, user, NULL);
+
+    /* Have to make this key a permanent copy for the hash */
+
+    klen = strlen(keybuf);
+    item.key = malloc(klen + 1);
+    if (!item.key) {
+        return Error(0, 0,
+           "%tout of memory for user ng processing\n");
+    }
+    strcpy(item.key, keybuf);
+    item.data = passed? YES: NO;
+    if (!s_hsearch(HS_USER, item, ENTER)) {
+        return Error(0, 0,
+           "%thash set failure for user ng processing\n");
+    }
+    return passed;
+}
+
+static int created_host_table = 0;
+static netgrp_h_compare(pattern, host)
+  char *pattern;
+  char *host;
+{
+    ENTRY item, *found_item;
+    int passed, klen;
+
+    if (!created_host_table) {
+        if (!s_hcreate(HS_HOST, 1000)) {
+	    return Error(0, 0,
+		"%tCouldn't allocate hash table for host ng processing\n");
+	}
+        created_host_table = 1;
+    }
+    /* Catenate pattern and user for the key such that
+     * "abc" + "de" is not the same as "ab" + "cde".
+     * Use "\n", as that will have been stripped from input before we get here.
+     */
+    if (snprintf(keybuf, MAX_KEYLEN, "%s\n%s", pattern, host) >= MAX_KEYLEN) {
+         return Error(0, 0,
+             "%tpattern and host too large for host ng processing\n");
+    }
+
+    item.key = keybuf;
+    if (found_item = s_hsearch(HS_HOST, item, FIND)) {
+        return found_item->data == YES? 1: 0;
+    }
+    passed = innetgr(pattern, host, NULL, NULL);
+
+    /* Have to make this key a permanent copy for the hash */
+
+    klen = strlen(keybuf);
+    item.key = malloc(klen + 1);
+    if (!item.key) {
+        return Error(0, 0,
+           "%tout of memory for host ng processing\n");
+    }
+    strcpy(item.key, keybuf);
+    item.data = passed? YES: NO;
+    if (!s_hsearch(HS_HOST, item, ENTER)) {
+        return Error(0, 0,
+           "%thash set failure for host ng processing\n");
+    }
+    return passed;
+}
+
 #else
 #define netgrp_u_compare(p, u) 0
 #define netgrp_h_compare(p, h) 0
@@ -59,14 +163,16 @@ char *pat;	/* pattern that value must match */
     /* Environment variables are always checked with s_re_comp/s_re_exec:
      * the patterns are fixed internally, not supplied by the user.
      */
-    if (s_re_comp(pat))
+    if (s_re_comp(pat)) {
 	return Error(0, 0,
 	    "%t\n\tcheckenv(): couldn't compile pattern `%-.500s'.\n", pat);
+    }
 
-    if (s_re_exec(value) != 1)
+    if (s_re_exec(value) != 1) {
 	return Error(0, 0,
 	    "checkenv(): $%.100s (=%.100s) doesn't match pattern %-.500s.\n",
 			name, value, pat);
+    }
 
     /* Limit the value to a reasonable length (MAXENVLEN chars) */
     len = strlen(value);
@@ -194,13 +300,15 @@ char *pat;	/* pattern that value must match */
 	    "\tcheck_value args: value=\"%s\"; pat=\"%s\"\n",
 	    value, pat);
     
-    if (s_re_comp(pat))
+    if (s_re_comp(pat)) {
 	return Error(0, 0,
 	    "%t\n\tcheck_value(): couldn't compile pattern `%-.500s'.\n", pat);
+    }
 
-    if (s_re_exec(value) != 1)
+    if (s_re_exec(value) != 1) {
 	return Error(0, 0,
 	    "check_value(): value doesn't match pattern %-.500s.\n", pat);
+    }
     return 0;
 }
 
@@ -1238,9 +1346,11 @@ char *salt;
     if (iscomsec()) {
 	#if (HPUX_MAJOR == 10)
 	    return bigcrypt(buf, salt);
-	#else
-	    return crypt(buf, salt);
 	#endif
+	#if (HPUX_MAJOR == 11)
+	    return bigcrypt(buf, salt);
+	#endif
+	return crypt(buf, salt);
     } else {
 	return crypt(buf, salt);
     }
@@ -1377,8 +1487,13 @@ int isglobal;		/* Is this a per-command or global condition? */
      * Form the globlist and pass along...
      */
     /* Do brace globbing */
-    if ((i=globbraces(s, 1, &globlist)) != 0)		/* Local Condition */
+    if ((i=globbraces(s, 1, &globlist)) != 0) {
+	/* Local Condition */
 	return Error(0, 0, "%tMissing `%c'.\n", i);
+    }
+    if (balancedbraces(s) != 0) {
+	return Error(0, 0, "%tUnbalanced braces in %s.\n", s);
+    }
 
     if (STRMATCH3("time", condition, s-1)) {
 	tl = isglobal ? &globalinfo.timeafter : &localinfo.time;
@@ -1562,7 +1677,7 @@ char *pattern;
     if (do_glob != 0) {
 	/* Do brace globbing on the pattern */
 	wrap = (do_glob > 1) ? 1 : 0;
-	if ((i=globbraces(pattern, wrap, &patlist)) != 0) {	/* MatchPat */
+	if ((i=globbraces(pattern, wrap, &patlist)) != 0) {
 	    Error(0, 0, "%tMissing `%c'.\n", i);
 	    return match;
 	}
@@ -1571,6 +1686,10 @@ char *pattern;
 	pat1list[1] = NULL;
 	patlist = pat1list;
     }
+    if (balancedbraces(pattern) != 0) {
+	Error(0, 0, "%tUnbalanced braces in %s.\n", pattern);
+	return match;
+    }
 
     for (ipat=0; (tok=patlist[ipat]); ipat++) {
 	strcpy(tokbuf, tok);
@@ -1617,28 +1736,32 @@ char *token;		/* user/group/host pattern */
 
     /* Split into user:group@host; check host part first (if it exists) */
     if ((hostpat = strchr(token, '@'))) {
-	if (hostpat[1] == 0)
-	return Error(0, 0, "%tMissing hostname in pattern `%s'.\n", origtext);
+	if (hostpat[1] == 0) {
+	    return Error(0, 0,
+	    	"%tMissing hostname in pattern `%s'.\n", origtext);
+	}
 	*hostpat++ = 0;
  
 	match = -1;
 	if (hostpat[0] == '+') {
 
 #ifdef HAVE_INNETGR
-	    if (hostpat[1] == 0)
+	    if (hostpat[1] == 0) {
 		return Error(0, 0,
 			"%tMissing netgroupname in pattern `%s'.\n", origtext);
+	    }
 	    match = check_host(hostpat, userinfo.hostname);
 #else
-		return Error(0, 0, 
+	    return Error(0, 0, 
 	"%thostnames may not begin with `+' since this super() was compiled\n\
 without -DHAVE_INNETGR.\n");
 #endif
 	} else {
 	    strtolower(hostpat);
 	    anchor(hostpat, chkbuf);    /* Force all matches to be anchored */
-	    if ((*pat_compile)(chkbuf) != NULL)
+	    if ((*pat_compile)(chkbuf) != NULL) {
 		return Error(0, 0, "%tbad host pattern: `%s'.\n", origtext);
+	    }
 	}
  
 	if (match == -1)
@@ -1673,30 +1796,34 @@ without -DHAVE_INNETGR.\n");
 	    *grouppat = '\0';			/* pat is "uuu:" */
 	grouppat = "^.*$";
     }
-    if (strchr(grouppat, ':'))
+    if (strchr(grouppat, ':')) {
 	return Error(0, 0, "%t\n\tGroup pattern `%s' contains a colon!\n",
 			grouppat);
+    }
 
-    if (globalinfo.group_slash == 0 && strchr(grouppat, '/'))
-		return Error(0, 0, "%t\n\tFormat error in super.tab file: \
+    if (globalinfo.group_slash == 0 && strchr(grouppat, '/')) {
+	return Error(0, 0, "%t\n\tFormat error in super.tab file: \
 group pattern `%s' contains a slash.\n\
 \tPerhaps you meant to use Cmd::Filename, but forgot one colon,\n\
 \tso it looks like User:Group?  If you really need to allow\n\
 \tslashes in group patterns, use global option group_slash=y.\n", grouppat);
+    }
 
 
 #ifdef HAVE_INNETGR
     if (userpat[0] == '+') {
-	if (userpat[1] == 0)
+	if (userpat[1] == 0) {
 	    return Error(0, 0,
 			"%tMissing netgroupname in pattern `%s'.\n", origtext);
+	}
 	match = netgrp_u_compare(&userpat[1], userinfo.caller.pw_name);
     } else
 #endif
     {
 	anchor(userpat, chkbuf);		/* Anchor all matches */
-	if ((*pat_compile)(chkbuf) != NULL)
+	if ((*pat_compile)(chkbuf) != NULL) {
 	    return Error(0, 0, "%t\n\tbad user pattern: `%s'.\n", origtext);
+	}
 	match = (*pat_compare)(userinfo.caller.pw_name);
 #ifdef MATCH_DECIMAL_UID
 	if (match != 1) {
@@ -1740,6 +1867,9 @@ group pattern `%s' contains a slash.\n\
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Determines if user's group matches a group pattern. */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
+
+static int created_group_table = 0;
+
 int
 ingroup(user, gid, gp_pat)
 char *user;
@@ -1770,9 +1900,36 @@ char *gp_pat;	/* pattern to match */
     SETGRENT_T setgrent();
     void endgrent();
 
+    ENTRY item, *found_item;
+    int passed, klen;
+
     if ((*pat_compile)(gp_pat) != (char *)0 )
 	return -1;
 
+    if (!created_group_table) {
+        if (!s_hcreate(HS_GROUP, 1000)) {
+	    return Error(0, 0,
+		"%tCouldn't allocate hash table for group processing\n");
+	}
+        created_group_table = 1;
+    }
+    /* Catenate user, gid and gp_pat for the key such that
+     * "abc" + 10 + "de" is not the same as "ab" + 1 + "0cde".
+     * Use "\n", as that will have been stripped from input before we get here.
+     */
+    if (snprintf(keybuf, MAX_KEYLEN, "%s\n%d\n%s", user, gid, gp_pat)
+             >= MAX_KEYLEN) {
+         return Error(0, 0,
+             "%tuser/gid/gp_pat too large for user ng processing\n");
+    }
+
+    item.key = keybuf;
+    if (found_item = s_hsearch(HS_GROUP, item, FIND)) {
+        return found_item->data == YES? 1: 0;
+    }
+
+    passed = 0;     /* Assume the worst */
+
     /* Search group file for groups user is in.  For each group of which
      * the user is a member, test a match to the pattern.
      */
@@ -1797,7 +1954,9 @@ char *gp_pat;	/* pattern to match */
 	if ((*pat_compare)(gp->gr_name) == 1) {
 	    /* successful match -- user is in a group that matches gp_pat */
 	    endgrent();
-	    return 1;
+	    /* return 1; */
+            passed = 1;
+            goto found_it;
 	}
 #ifdef MATCH_DECIMAL_GID
 	else {
@@ -1810,7 +1969,9 @@ char *gp_pat;	/* pattern to match */
 	    if ((*pat_compare)(buf) == 1){
 		/* successful match -- user is in a group that matches gp_pat */
 		endgrent();
-		return 1;
+		/* return 1; */
+                passed = 1;
+                goto found_it;
 	    }
 	}
 #endif
@@ -1827,11 +1988,31 @@ char *gp_pat;	/* pattern to match */
     (void) sprintf(buf, "%d", gid);
     if ((*pat_compare)(buf) == 1){
 	endgrent();
-	return 1;
+	/* return 1; */
+        passed = 1;
+        goto found_it;
     }
 #endif
 
     endgrent();
-    return 0;
+    /* return 0; */
+
+found_it:
+
+    /* Have to make this key a permanent copy for the hash */
+
+    klen = strlen(keybuf);
+    item.key = malloc(klen + 1);
+    if (!item.key) {
+        return Error(0, 0,
+           "%tout of memory for group processing\n");
+    }
+    strcpy(item.key, keybuf);
+    item.data = passed? YES: NO;
+    if (!s_hsearch(HS_GROUP, item, ENTER)) {
+        return Error(0, 0,
+           "%thash set failure for group processing\n");
+    }
+    return passed;
 }
 
