@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: rsyslog.c,v 1.17 2004/04/30 17:00:58 will Exp $";
+static const char rcsid[] = "$Id: rsyslog.c,v 1.17 2004/04/30 17:00:58 will Exp will $";
 /*************************************************************************
 **
 ** Network version of syslog (syslog,openlog,closelog)
@@ -103,7 +103,7 @@ static struct {
 	(void) sprintf(msg,"(%d) ", getpid());
     if (*loginfo.ident)
 	(void) sprintf(msg+strlen(msg),"%s: ", loginfo.ident);
-    (void) vsprintf(msg+strlen(msg), fmt, args);
+    (void) vsnprintf(msg+strlen(msg), sizeof(msg)-strlen(msg), fmt, args);
     va_end(args);
 
     /*
