@@ -510,6 +510,7 @@ int	canonicalize_hostname P__(( char *buf, int len));
 int	check_arglistlen P__(( char **argv ));
 int	check_auth P__(( char *cmd ));
 int	check_owner P__(( void ));
+int	check_rpath P__((char *r_path, char *path));
 void	check_stdio P__((void));
 int	check_value P__(( char *value, char *pat ));
 int	check_var_value P__(( void ));
@@ -531,7 +532,8 @@ int	daynum P__(( int unixtime ));
 void	debug_print P__((char *path, char **arglist, char **envp, int n_bltin));
 void	debug_hello P__((void));
 int	do_options P__((int argc, char **argv, int *help, int *vers,
-		    int *verbose, int *printvars, char **owned_by_file));
+		    int *verbose, int *printvars,
+		    char **owned_by_file, char **required_path));
 char*	do_variables P__(( char *str));
 char*	docrypt P__((char *key, char *salt));
 char*	ends P__(( char *s1, char *s2 ));
